<?php

namespace Bluebirdday\TranssmartSmartConnect\Model\Config\Config;

use \Magento\Config\Model\Config\Structure as OriginalStructure;
use Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile\AllowedCountry;
use Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository;

class StructurePlugin
{
    /**
     * @var ProfileRepository
     */
    private $profileRepository;

    /**
     * SectionPlugin constructor.
     * @param ProfileRepository $profileRepository
     */
    public function __construct(
        ProfileRepository $profileRepository
    ) {
        $this->profileRepository = $profileRepository;
    }

    /***
     * Collects config paths and their structure paths from configuration files.
     * Because booking_profiles are dynamically created they are not declared in any system.xml
     * This plugin dynamically adds them to make sure transsmart_profile configuration will be saved in the core_config_data table
     *
     * @param OriginalStructure $subject
     * @param $result
     * @return mixed
     */
    public function afterGetFieldPaths(
        \Magento\Config\Model\Config\Structure $subject,
        $result
    ) {
        return array_merge($result, $this->getDynamicConfigGroups());
    }


    /**
     * Get Dynamic Config Groups
     *
     * @return array
     */
    private function getDynamicConfigGroups()
    {
        $profiles = $this->profileRepository->getList();
        $dynamicConfigGroups = [];

        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile $profile */
        foreach ($profiles as $index => $profile) {
            $dynamicConfigFields = [];

            $allowSpecific = implode('/', [AllowedCountry::CONFIG_TRANSSMART_PROFILES_SECTION_ID, $profile->getCode(), 'sallowspecific']);
            $specificCountry = implode('/', [AllowedCountry::CONFIG_TRANSSMART_PROFILES_SECTION_ID, $profile->getCode(), 'specificcountry']);
            $dynamicConfigFields[$allowSpecific] = [$allowSpecific];
            $dynamicConfigFields[$specificCountry] = [$specificCountry];

            $dynamicConfigGroups = array_merge($dynamicConfigGroups, $dynamicConfigFields);
        }

        return $dynamicConfigGroups;
    }
}
