<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Location;

/**
 * Location Request Data Container
 *
 * @package Bluebirdday\TranssmartSmartConnect\Model
 */
class Request extends \Bluebirdday\TranssmartSmartConnect\Model\Data\Container
{
    protected $requiredFields = [
        'city' => 'city',
        'zip_code' => 'zipCode',
        'street' => 'street',
        'house_number' => 'houseNumber',
        'country_from' => 'countryFrom',
        'country_to' => 'countryTo',
        'provider' => 'provider',
        'limit' => 'limit',
    ];

    protected $optionalFields = [
        'house_number_ext' => 'houseNumberExt',
    ];

    /**
     * @return string
     */
    public function getStreet()
    {
        return $this->getData('street');
    }

    /**
     * @param string $value
     */
    public function setStreet($value)
    {
        $this->setData('street', $value);
    }

    /**
     * @return string|int
     */
    public function getHouseNumber()
    {
        return $this->getData('house_number');
    }

    /**
     * @param string|int $value
     */
    public function setHouseNumber($value)
    {
        $this->setData('house_number', $value);
    }

    /**
     * @return string
     */
    public function getHouseNumberExt()
    {
        return $this->getData('house_number_ext');
    }

    /**
     * @param string $value
     */
    public function setHouseNumberExt($value)
    {
        $this->setData('house_number_ext', $value);
    }

    /**
     * @return string
     */
    public function getZipCode()
    {
        return $this->getData('zip_code');
    }

    /**
     * @param string $value
     */
    public function setZipCode($value)
    {
        $this->setData('zip_code', $value);
    }

    /**
     * @return string
     */
    public function getCity()
    {
        return $this->getData('city');
    }

    /**
     * @param $value
     */
    public function setCity($value)
    {
        $this->setData('city', $value);
    }

    /**
     * Get two-letter origin country code
     *
     * @return string
     */
    public function getCountryFrom()
    {
        return $this->getData('country_from');
    }

    /**
     * @param $value
     */
    public function setCountryFrom($value)
    {
        $this->setData('country_from', $value);
    }

    /**
     * Get two-letter destination country code
     *
     * @return string
     */
    public function getCountryTo()
    {
        return $this->getData('country_to');
    }

    /**
     * @param $value
     */
    public function setCountryTo($value)
    {
        $this->setData('country_to', $value);
    }

    /**
     * Shipping provider(carrier) code or "ALL"
     *
     * @return string
     */
    public function getProvider()
    {
        return $this->getData('provider');
    }

    /**
     * @param $value
     */
    public function setProvider($value)
    {
        $this->setData('provider', $value);
    }

    /**
     * @return int|string
     */
    public function getLimit()
    {
        return $this->getData('limit');
    }

    /**
     * @param $value
     */
    public function setLimit($value)
    {
        $this->setData('limit', $value);
    }

    /**
     * {@inheritdoc}
     */
    protected function checkRequiredFields()
    {
        foreach ($this->requiredFields as $internal => $external) {
            if (null === $this->getData($internal)) {
                throw new \RuntimeException(__("Field '%1' is required. Please fill it in.", $external));
            }
        }
    }

    /**
     * Some customers immediately open the pick-up location modal without first filling in the checkout fields.
     * This check is to notify customers to first fill in the required fields without getting a specific error
     * on which field is empty.
     */
    protected function isCheckoutFilled()
    {
        $requiredAddressFields = [
            'city',
            'zip_code',
            'street',
            'country_to'
        ];

        $requiredFieldsNotSet = 0;
        foreach ($requiredAddressFields as $field) {
            if (null === $this->getData($field)) {
                $requiredFieldsNotSet++;
            }
        }

        if ($requiredFieldsNotSet >= 2) {
            throw new \RuntimeException(__("Please fill in all required address fields before selecting a pick-up location."));
        }
    }

    /**
     * Prepare data for URL output
     *
     * @return string
     */
    public function __toString()
    {
        $variables = [];

        $this->isCheckoutFilled();
        $this->checkRequiredFields();

        $fields = $this->requiredFields + $this->optionalFields;

        foreach ($fields as $internal => $external) {
            $variables[$external] = $this->getData($internal);
        }

        return http_build_query($variables);
    }
}
