<?php

namespace Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier;

use Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate as BaseTablerate;
use Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\RateQuery;

/**
 * Class Tablerate
 *
 * @package Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier
 */
class Tablerate extends BaseTablerate
{
    /**
     * @var BaseTablerate\RateQueryFactory
     */
    private $rateQueryFactory;

    /**
     * Tablerate constructor.
     *
     * @param \Magento\Framework\Model\ResourceModel\Db\Context $context
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $coreConfig
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\OfflineShipping\Model\Carrier\Tablerate $carrierTablerate
     * @param \Magento\Framework\Filesystem $filesystem
     * @param BaseTablerate\Import $import
     * @param BaseTablerate\RateQueryFactory $rateQueryFactory
     * @param null $connectionName
     */
    public function __construct(
        \Magento\Framework\Model\ResourceModel\Db\Context $context,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\App\Config\ScopeConfigInterface $coreConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\OfflineShipping\Model\Carrier\Tablerate $carrierTablerate,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\Import $import,
        \Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\RateQueryFactory $rateQueryFactory,
        $connectionName = null
    ) {
        $this->rateQueryFactory = $rateQueryFactory;
        parent::__construct($context, $logger, $coreConfig, $storeManager, $carrierTablerate, $filesystem, $import, $rateQueryFactory, $connectionName);
    }

    /**
     * Append default rate request with transsmart_bookingprofile_id filter.
     *
     * @param \Magento\Quote\Model\Quote\Address\RateRequest $request
     *
     * @return array|bool|mixed
     *
     * @see \Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate::getRate()
     */
    public function getRate(\Magento\Quote\Model\Quote\Address\RateRequest $request)
    {
        $connection = $this->getConnection();
        $select = $connection->select()->from($this->getMainTable());
        $rateQuery = $this->rateQueryFactory->create(['request' => $request]);

        $bindings = $rateQuery->getBindings();
        $rateQuery->prepareSelect($select);

        /* BBD getRate() edit START */
        $select->where('transsmart_bookingprofile_id = :transsmart_bookingprofile_id');
        $bindings[':transsmart_bookingprofile_id'] = $request->getData('transsmart_bookingprofile_id');
        /* BBD getRate() edit END */

        $result = $connection->fetchRow($select, $bindings);
        // Normalize destination zip code
        if ($result && $result['dest_zip'] == '*') {
            $result['dest_zip'] = '';
        }

        return $result;
    }

    /**
     * Return table rate array or false by rate request
     *
     * @param \Magento\Quote\Model\Quote\Address\RateRequest $request
     *
     * @return array|bool
     */
    public function getRates(\Magento\Quote\Model\Quote\Address\RateRequest $request)
    {
        $connection = $this->getConnection();

        $select = $connection
            ->select()
            ->from($this->getMainTable(), ['transsmart_bookingprofile_id'])
            ->where('transsmart_bookingprofile_id IS NOT NULL')
            ->distinct();
        $profileIds = $connection->fetchAssoc($select);
        $rates = [];
        foreach ($profileIds as $profileId) {
            $request->setData('transsmart_bookingprofile_id', $profileId['transsmart_bookingprofile_id']);
            $rates[] = $this->getRate($request);
        }

        return $rates;
    }

}

